
import pygame
import pygame.freetype
import random

SCREEN_WIDTH = 800 
SCREEN_HEIGHT = 800
ROAD_WIDTH = 80     # thickness of the road bands, and the distance the frog can jump
TRAVEL_AMOUNT = 10  # distance the frog travels per update

font = None



class Vehicle:
    def __init__(self, sprite, x, y):
        """
        Initialize the vehicle

        params
            sprite - (pygame.Surface)
            x - (int)
            y = (int)
        """
        self.sprite = sprite
        self.rect = self.sprite.get_rect()
        self.rect.x = x
        self.rect.y = y

    def update(self, dx):
        """
        Update the position of the vehicle

        params:
            dx - (int) the amount to move
        """
        self.rect.x += dx

    def display(self, surface):
        """
        Display the box
        
        params
            surface - (pygame.Surface)
        """
        surface.blit(self.sprite, (self.rect.x, self.rect.y))



class Road:
    def __init__(self, sprite, y, speed):
        """
        Initialize the Road

        This creates the Vehicles and sets them up. The object has the following properties:
            - y (y coordinate for the top of the road)
            - speed (the rate the vehicles travel on this road - the direction can be derived from the sign)
            - sprite_width (the width of the sprite used for the vehicles on this road)
            - vehicles (the vehicles that travel this road)

        params:
            sprite - (pygame.Surface)
            y - (int)
            speed - (int)
        """
        self.y = y
        self.speed = speed
        self.sprite_width = sprite.get_rect().width
        self.vehicles = []

        # add vehicles to the road
            
    

    def update(self):
        """
        Update the positions of the vehicles on the road

        This updates the position of the vehicles and then checks if the lead has left the screen. If it has, it is removed from the list, repositioned behind the last vehicle and added back to the list
        
        """
        
        # handle vehicle updates


    
    def display(self, surface):
        """
        Displays the road and its contents
        """

        pygame.draw.rect(surface, (127, 127, 127), (0, self.y, SCREEN_WIDTH, ROAD_WIDTH-10))
        for vehicle in self.vehicles:
            vehicle.display(surface)

    def collision(self, frog):
        """
        Takes in a frog and returns a boolean indicating if any of its vehicles hit the frog.

        params
            frog - (Frog)
        """
        if frog.rect.top  < self.y + ROAD_WIDTH and frog.rect.bottom > self.y:
            for vehicle in self.vehicles:
                if vehicle.rect.colliderect(frog.rect):
                    return True
        return False


class Scene:

    def __init__(self):
        """
        Initialize the scene

        Loads all of the various sprites and calls reset
        """
        self.vehicle_sprites = {"right": [], "left": []}
     
        for vehicle in ["car1", "car2", "car3", "truck", "firetruck"]:
            for direction in self.vehicle_sprites.keys():
                self.vehicle_sprites[direction].append(pygame.image.load(f"assets/{vehicle}-{direction}.png"))

        self.reset()
        
    def reset(self):
        """
        Initializes the elements of the scene

        Creates a Frog at the bottom and constructs all of the Roads
        """
        # uncomment when you have created a Frog
        # self.frog = Frog(SCREEN_WIDTH // 2, SCREEN_HEIGHT - ROAD_WIDTH + 20)
        self.message = ""
        self.roads = []
        for i in range(1, 8):
            direction = ["right", "left"][i % 2]
            sprite = random.choice(self.vehicle_sprites[direction])
            if direction == "right":
                speed = random.randint(1, 3)
            else:
                speed = random.randint(-3, -1)
            self.roads.append(Road(sprite, i * ROAD_WIDTH, speed))


     
    def update(self):
        """
        Handles the update events for all of the elements

        Also checks for the two game ending events
        """
        # uncomment when you have created a Frog
        # for road in self.roads:
        #     if road.collision(self.frog):
        #         self.message = "SPLAT!"
        #         return
     
        # self.frog.update()

        # if self.frog.rect.bottom < ROAD_WIDTH:
        #     self.message = "You win!"
        #     return

        for road in self.roads:
            road.update()
        

    def display(self, surface):
        """
        Call display on all of the elements of the scene

        params
            surface - (pygame.Surface)
        """
        
        for road in self.roads:
            road.display(surface)
        
        # uncomment when yuo have created a Frog
        # self.frog.display(surface)
        if self.message:
            pygame.draw.rect(surface, "black", (SCREEN_WIDTH // 4, SCREEN_HEIGHT // 2 - 50, SCREEN_WIDTH // 2, 100))
            space = font.get_rect(self.message)
            center = ((SCREEN_WIDTH - space.width)//2, (SCREEN_HEIGHT - space.height) // 2)
            font.render_to(surface, center, self.message, 'white')

    def key_event(self, key):
        """
        Check the key event and move the frog appropriately

        param:
            key - (int)
        """
        # uncomment when yuo have created a Frog
        # if key == pygame.K_UP:
        #     self.frog.jump('forward')
        # elif key == pygame.K_DOWN:
        #     self.frog.jump('backward')
        # elif key == pygame.K_LEFT:
        #     self.frog.jump('left')
        # elif key == pygame.K_RIGHT:   
        #     self.frog.jump('right')

       
 



if __name__ == "__main__":
    
    pygame.init()
    font = pygame.freetype.SysFont(None, 48)
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), pygame.HWSURFACE | pygame.DOUBLEBUF)
    clock = pygame.time.Clock()
    running = True
    scene = Scene()

    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running  = False
            elif event.type == pygame.TEXTINPUT:
                if event.text == "r":
                    scene.reset()
            elif event.type == pygame.KEYDOWN:
                scene.key_event(event.key)
                

   
                
        screen.fill("white")
      
        # update display here
        scene.update()
        scene.display(screen)

        pygame.display.flip()
        clock.tick(60) # rate limit the refresh

 
    pygame.quit()
